// Fighter logic

#include "FighterCommon.as"
#include "ThrowCommon.as"
#include "Knocked.as"
#include "Hitters.as"
#include "RunnerCommon.as"
#include "ShieldCommon.as";
#include "Help.as";
#include "BombCommon.as";

const int PICKUP_COOLDOWN=	15;
const int STAB_DELAY=	10;
const int STAB_TIME=	22;
const int SHOTGUN_DELAY=	25;
const int PISTOL_DELAY=		5;


void onInit(CBlob@ this)
{
	FighterInfo fighter;
	this.set("fighterInfo", @fighter);
	RunnerMoveVars runner;
	this.set("runnerMoveVars", @runner);

	this.set_u8("currentWeapon",0);
	this.set_f32("gib health", -3.0f);
	this.Tag("player");
	this.Tag("flesh");

	this.set_Vec2f("inventory offset", Vec2f(0.0f, 0.0f));

	//no spinning
	this.getShape().SetRotationsAllowed(false);
	this.addCommandID("fireWeapon");
	this.getShape().getConsts().net_threshold_multiplier=	0.5f;

	this.getCurrentScript().runFlags |= Script::tick_not_attached;
	this.getCurrentScript().removeIfTag=	"dead";
	this.push("names to activate", "keg");
	
	if(this.hasBlob("shotgun",1)){
		fighter.weapon_fireDelayMax=SHOTGUN_DELAY;
	}else{
		fighter.weapon_fireDelayMax=PISTOL_DELAY;
	}
	runner.walkFactor*=3;
}

void onSetPlayer(CBlob@ this, CPlayer@ player)
{
	if(player !is null)
	{
		player.SetScoreboardVars("ScoreboardIcons.png", 2, Vec2f(16, 16));
	}
}

void ManageWeapon(CBlob@ this, FighterInfo@ fighter, RunnerMoveVars@ moveVars)
{
	CSprite@ sprite=	this.getSprite();
	bool isMyPlayer=	this.isMyPlayer();
	s8 weapon_fireDelay=fighter.weapon_fireDelay;
	Vec2f pos=	this.getPosition();

	if(weapon_fireDelay>0){
		weapon_fireDelay--;
		if(weapon_fireDelay<=0){
			weapon_fireDelay=0;
		}
	}
	if(weapon_fireDelay==0 && this.isKeyPressed(key_action1) && !this.isKeyPressed(key_inventory))
	{
		if(!isMyPlayer){
			sprite.SetEmitSoundVolume(0.5f);
		}
		ClientFire(this);
		weapon_fireDelay=	fighter.weapon_fireDelayMax;
	}
	if(isMyPlayer){
		if(!getHUD().hasButtons()){
			int frame=	0;
			if(weapon_fireDelay>0){
				frame=	1+float(weapon_fireDelay)/float(fighter.weapon_fireDelayMax)*7;
			}
			getHUD().SetCursorFrame(frame);
		}
		if(this.isKeyJustPressed(key_action3)){
			client_SendThrowOrActivateCommand(this);
		}
	}
	fighter.weapon_fireDelay=	weapon_fireDelay;
}

void onTick(CBlob@ this)
{
	FighterInfo@ fighter;
	if(!this.get("fighterInfo", @fighter))
	{
		return;
	}

	if(getKnocked(this)>0){
		return;
	}

	// vvvvvvvvvvvvvv CLIENT-SIDE ONLY vvvvvvvvvvvvvvvvvvv

	if(!getNet().isClient()) return;

	if(this.isInInventory()) return;

	RunnerMoveVars@ moveVars;
	if(!this.get("moveVars", @moveVars))
	{
		return;
	}

	ManageWeapon(this, fighter, moveVars);
}

bool canSend(CBlob@ this)
{
	return(this.isMyPlayer() || this.getPlayer() is null || this.getPlayer().isBot());
}

void ClientFire(CBlob@ this)
{
	//time to fire!
	if(canSend(this))  // client-logic
	{
		Vec2f firePos=	this.getPosition()+Vec2f(0.0f,-6.0f);
		Vec2f aimPos=	this.getAimPos()+Vec2f(0.0f,-2.0f);
		f32 bulletSpeed=FighterParams::shoot_max_vel;
		
		// player or bot
		Vec2f fireVel=	(aimPos-firePos);
		fireVel.Normalize();
		fireVel*=bulletSpeed;
		CBitStream params;
		params.write_Vec2f(firePos);
		params.write_Vec2f(fireVel);
		
		this.SendCommand(this.getCommandID("fireWeapon"), params);
	}
}
CBlob@ CreateBullet(CBlob@ this, Vec2f bulletPos, Vec2f bulletVel)
{
	CBlob@ bullet=	server_CreateBlobNoInit("bullet");
	if(bullet !is null)
	{
		bullet.Init();
		bullet.IgnoreCollisionWhileOverlapped(this);
		bullet.SetDamageOwnerPlayer(this.getPlayer());
		bullet.server_setTeamNum(this.getTeamNum());
		bullet.setPosition(bulletPos);
		bullet.setVelocity(bulletVel);
	}
	return bullet;
}

void onCommand(CBlob@ this, u8 cmd, CBitStream @params)
{
	if(cmd == this.getCommandID("fireWeapon"))
	{
		Vec2f firePos=	params.read_Vec2f();
		Vec2f fireVel=	params.read_Vec2f();

		FighterInfo@ fighter;
		if(!this.get("fighterInfo", @fighter))
		{
			return;
		}
		if(this.hasBlob("shotgun",1))
		{
			if(getNet().isServer())
			{
				uint seed=	getGameTime()*this.getNetworkID();
				Random@ r=	Random(seed);
				for(int i=0;i<8; i++)
				{
					Vec2f rotated=	(fireVel*((r.NextFloat()*0.25)+0.75)).RotateBy((r.NextFloat()*5)-2.5,Vec2f());
					CBlob@ bullet=	CreateBullet(this, firePos,rotated); //*((r.NextFloat()*0.5)+0.5)
				}
			}
			this.getSprite().PlaySound("/ShotgunFire.ogg");
		}
		else
		{
			if(getNet().isServer())
			{
				CreateBullet(this, firePos, fireVel);
			}
			this.getSprite().PlaySound("/RifleFire1.ogg");
		}
	}
}

void onCreateInventoryMenu(CBlob@ this, CBlob@ forBlob, CGridMenu @gridmenu)
{
	this.ClearGridMenusExceptInventory();
}

void onAddToInventory(CBlob@ this, CBlob@ blob)
{
	string itemname=	blob.getName();
	CInventory@ inv=	this.getInventory();
	
	FighterInfo@ fighter;
	if(!this.get("fighterInfo", @fighter)){
		return;
	}
	if(this.hasBlob("shotgun",1)){
		fighter.weapon_fireDelayMax=SHOTGUN_DELAY;
	}else{
		fighter.weapon_fireDelayMax=PISTOL_DELAY;
	}
}
void onRemoveFromInventory(CBlob@ this, CBlob@ blob)
{
	FighterInfo@ fighter;
	if(!this.get("fighterInfo", @fighter)){
		return;
	}
	fighter.weapon_fireDelayMax=PISTOL_DELAY;
}

void onHitBlob(CBlob@ this, Vec2f worldPoint, Vec2f velocity, f32 damage, CBlob@ hitBlob, u8 customData)
{
	if(customData == Hitters::stab)
	{
		if(damage > 0.0f)
		{
			if(blockAttack(hitBlob, velocity, 0.0f))
			{
				this.getSprite().PlaySound("/Stun", 1.0f, this.getSexNum() == 0 ? 1.0f : 2.0f);
				SetKnocked(this, 30);
			}
		}
	}
}

